import moment from 'moment';

/**
 * 计算字符串维度
 * @param {*} dimensionKey
 * @param {*} data
 * @returns
 */
const _getStringDimensionItem = (value, hadDimensions) => {
  if (!hadDimensions[value]) {
    hadDimensions[value] = true;
    return {
      label: value,
      value: value,
    };
  }
};
export const getStringDimension = (dimensionKey, data) => {
  let dimensions = [];
  const hadDimensions = {};
  data.forEach(item => {
    const itemValue = _getStringDimensionItem(item[dimensionKey], hadDimensions);
    if (itemValue) {
      dimensions.push(itemValue);
    }
  });
  return {
    data: dimensions,
    parseValue: null,
  };
};

/**
 * 计算数字维度-范围
 * @param {*} dimensionKey 维度字段
 * @param {*} data 数据
 * @returns
 */
const _getNumberRangeDimension = (dimensionKey, data) => {
  let dimensions = [];
  let min = 10000000000000;
  let max = 0;
  data.forEach(item => {
    if (item[dimensionKey]) {
      if (item[dimensionKey] < min) {
        min = item[dimensionKey];
      }
      if (item[dimensionKey] > max) {
        max = item[dimensionKey];
      }
    }
  });
  const count = parseInt((max - min) / 2, 10);
  dimensions.push({
    label: `${min} - ${min + count}`,
    value: {
      min: min,
      max: min + count,
    },
  });
  dimensions.push({
    label: `${min + count} - ${max}`,
    value: {
      min: min + count,
      max: max,
    },
  });
  return {
    data: dimensions,
    parseValue: null,
  };
};
export const getNumberRangeDimension = _getNumberRangeDimension;

/**
 * 计算日期维度
 * @param {*} dimensionKey 维度字段
 * @param {*} data  数据
 * @param {*} dateType 日期类型： 年、季、月、周、日
 * @returns
 */
const _getDateDimension = (value, dateType, hadDimensions) => {
  let dimensionItemValue;
  if (dateType === '年') {
    dimensionItemValue = moment(value).format('YYYY');
  } else if (dateType === '月') {
    dimensionItemValue = moment(value).format('YYYY-MM');
  } else if (dateType === '日') {
    dimensionItemValue = moment(value).format('YYYY-MM-DD');
  } else if (dateType === 'MM/DD') {
    dimensionItemValue = moment(value).format('MM/DD');
  }
  if (dateType === '季') {
    dimensionItemValue = `${moment(value).format('YYYY')} Q${moment(value).format('Q')}`;
  }

  if (!hadDimensions[dimensionItemValue]) {
    hadDimensions[dimensionItemValue] = true;
    return {
      label: dimensionItemValue,
      value: dimensionItemValue,
    };
  }
};
export const getDateDimension = (dimensionKey, data, dateType) => {
  let dimensions = [];
  const hadDimensions = {};
  const years = [];
  const months = [];
  const days = [];
  let type;
  data.forEach(item => {
    if (item[dimensionKey]) {
      const year = moment(item[dimensionKey]).format('YYYY');
      const day = moment(item[dimensionKey]).format('YYYY-MM-DD');
      const month = moment(item[dimensionKey]).format('YYYY-MM');
      if (!hadDimensions[year]) {
        years.push({
          label: year,
          value: year,
        });
        hadDimensions[year] = true;
      }
      if (!hadDimensions[month]) {
        months.push({
          label: month,
          value: month,
        });
        hadDimensions[month] = true;
      }
      if (!hadDimensions[day]) {
        days.push({
          label: day,
          value: day,
        });
        hadDimensions[day] = true;
      }
    }
  });
  if (years.length > 5) {
    dimensions = years;
    type = '年';
  } else if (months.length > 5) {
    dimensions = months;
    type = '月';
  } else {
    dimensions = days;
    type = '日';
  }
  return {
    data: dimensions,
    dateType: type,
  };
};

const mathFn = {
  string: getStringDimension,
  number: getStringDimension,
  numberRange: getNumberRangeDimension,
  date: getDateDimension,
};

export const parseDimensionAndData = (
  { autoDimensions, structureKeyName, dimensions = [], dateType, structureType },
  { measureKey, collectType, tofixed = 2, math = '' },
  data = [],
) => {
  let newData = [];
  let dimensionValueObj = {},
    key;
  let dimensionValueCount = {};
  let dimensionDiffValueCount = {};
  let dimension = [];
  const hadValue = {};
  const newDimensionData = autoDimensions ? [] : dimensions;
  const hadDimensionData = {};
  let integer = 1000000;

  data.forEach(item => {
    if (item[structureKeyName] && (item[measureKey] || item[measureKey] === 0)) {
      let dimensionValue = item[structureKeyName];

      // 当"自动计算维度"为true时，遍利数据中的维度值，计数维度数据
      if (autoDimensions) {
        let dimensionItem;
        if (structureType === 'date') {
          dimensionItem = _getDateDimension(dimensionValue, dateType, hadDimensionData);
        } else {
          dimensionItem = _getStringDimensionItem(dimensionValue, hadDimensionData);
        }

        if (dimensionItem) {
          newDimensionData.push(dimensionItem);
          dimensionValue = dimensionItem.value;
        }
      }

      // 通过维度统计度量的总和、计数、计数(不同)
      dimensionValueObj[dimensionValue] =
        (dimensionValueObj[dimensionValue] || 0) + (Number(item[measureKey]) * integer || 0);
      if (item[measureKey] || item[measureKey] === 0) {
        dimensionValueCount[dimensionValue] = (dimensionValueCount[dimensionValue] || 0) + 1;

        if (!hadValue[`${dimensionValue}_${item[measureKey]}`]) {
          dimensionDiffValueCount[dimensionValue] =
            (dimensionDiffValueCount[dimensionValue] || 0) + 1;
          hadValue[`${dimensionValue}_${item[measureKey]}`] = true;
        }
      }
    }
  });

  // 求统计度量的平均
  if (collectType === '平均值') {
    for (key in dimensionValueObj) {
      dimensionValueObj[key] = dimensionValueObj[key] / dimensionValueCount[key];
    }
  }

  newDimensionData.map(item => {
    dimension.push(item.label);
    if (collectType === '计数') {
      newData.push(dimensionValueCount[item.value]);
    } else if (collectType === '计数(不同)') {
      newData.push(dimensionDiffValueCount[item.value]);
    } else if (dimensionValueObj[item.value] || dimensionValueObj[item.value] === 0) {
      let resultValue = dimensionValueObj[item.value] / integer;
      if (math) {
        let newMath = math;
        newMath = newMath.replace(/\[[A-Za-z0-9_]+\]/g, function(string) {
          return resultValue;
        });
        try {
          resultValue = new Function(`return ${newMath}`)();
        } catch (e) {
          //
        }
      }
      newData.push(resultValue.toFixed(tofixed));
    }
  });

  return {
    data: newData || [],
    dimension,
  };
};

export const getData = (nextPropsData, nextPropsDataKeys) => {
  if (nextPropsDataKeys && nextPropsDataKeys.length) {
    const data = nextPropsData && nextPropsData[0] ? nextPropsData[0] : {},
      newData = {},
      newDataKeys = [];

    nextPropsDataKeys.forEach(item => {
      newData[item.keyName] = data[item.keyName] || '';
      newDataKeys.push(item.keyName);
    });
    return {
      data: newData,
      dataKeys: newDataKeys,
    };
  }
  return null;
};

export const parseNumber = (num, decimals, separator) => {
  if(!(decimals || separator)) {
    return num;
  }

  let newNum = num || 0;
  newNum = newNum + '';
  if(decimals) {


    if(newNum.indexOf('.') > 0) {
      newNum = (newNum + '000000000').substr(0, newNum.indexOf('.') + 1 + decimals);
    }
    else {
      newNum = (newNum + '.000000000').substr(0, newNum.length + 1 + decimals);
    }
  }

  let numArr = ['', ''];
  if(newNum.indexOf('.') >= 0) {
    numArr = newNum.split('.');
  }
  else {
    numArr[0] = newNum
  }

  var result = [],
    counter = 0;
  numArr[0] = (numArr[0] || 0).toString().split('');
  for (var i = numArr[0].length - 1; i >= 0; i--) {
    counter++;
    result.unshift(numArr[0][i]);
    if (!(counter % 3) && i !== 0) {
      result.unshift(',');
    }
  }
  return numArr[1] ? [result.join(''), numArr[1]].join('.') : result.join('');
};
